package org.drools.examples.manners;

/*
 * $Id: MannersJSR94.java,v 1.3 2004/11/12 17:11:15 simon Exp $
 *
 * Copyright 2002 (C) The Werken Company. All Rights Reserved.
 *
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided that the
 * following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright statements and
 * notices. Redistributions must also contain a copy of this document.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * 3. The name "drools" must not be used to endorse or promote products derived
 * from this Software without prior written permission of The Werken Company.
 * For written permission, please contact bob@werken.com.
 *
 * 4. Products derived from this Software may not be called "drools" nor may
 * "drools" appear in their names without prior written permission of The Werken
 * Company. "drools" is a registered trademark of The Werken Company.
 *
 * 5. Due credit should be given to The Werken Company.
 * (http://drools.werken.com/).
 *
 * THIS SOFTWARE IS PROVIDED BY THE WERKEN COMPANY AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE WERKEN COMPANY OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

import org.drools.jsr94.rules.RuleServiceProviderImpl;

import javax.rules.InvalidRuleSessionException;
import javax.rules.RuleRuntime;
import javax.rules.RuleServiceProvider;
import javax.rules.RuleServiceProviderManager;
import javax.rules.StatelessRuleSession;
import javax.rules.admin.LocalRuleExecutionSetProvider;
import javax.rules.admin.RuleAdministrator;
import javax.rules.admin.RuleExecutionSet;
import java.io.IOException;
import java.io.InputStream;
import java.util.List;

/**
 * Uses the JSR-94 RuleServiceProviderImpl for Drools to solve the Miss Manners
 * problem.
 */
public class MannersJSR94 extends MannersBase
{
    /** Drools <code>RuleServiceProvider</code> URI. */
    private static final String  RULE_SERVICE_PROVIDER = "http://drools.org/";

    private RuleAdministrator    ruleAdministrator;

    private StatelessRuleSession statelessRuleSession;

    public static void main(String[] args) throws Exception
    {
        new MannersJSR94( args ).run( );
    }

    protected MannersJSR94(String[] args)
    {
        super( args );
    }

    /**
     * Setup the test case.
     */
    protected void setUp() throws Exception
    {
        // obtain the RuleServiceProvider
        RuleServiceProviderManager.registerRuleServiceProvider( RULE_SERVICE_PROVIDER,
                                                                RuleServiceProviderImpl.class );
        RuleServiceProvider ruleServiceProvider = RuleServiceProviderManager.getRuleServiceProvider( RULE_SERVICE_PROVIDER );
        ruleAdministrator = ruleServiceProvider.getRuleAdministrator( );

        // load the rules and register them
        LocalRuleExecutionSetProvider ruleSetProvider = ruleAdministrator.getLocalRuleExecutionSetProvider( null );
        InputStream rules = this.getClass( ).getResourceAsStream( ruleUri );
        RuleExecutionSet ruleExecutionSet = ruleSetProvider.createRuleExecutionSet( rules, null );
        ruleAdministrator.registerRuleExecutionSet( ruleUri, ruleExecutionSet, null );

        RuleRuntime ruleRuntime = ruleServiceProvider.getRuleRuntime( );
        statelessRuleSession = ( StatelessRuleSession ) ruleRuntime.createRuleSession( ruleUri,
                                                                                       null,
                                                                                       RuleRuntime.STATELESS_SESSION_TYPE );
    }

    /**
     * Tear down the test case
     */
    protected void tearDown() throws Exception
    {
        statelessRuleSession.release( );
        ruleAdministrator.deregisterRuleExecutionSet( ruleUri, null );
    }

    protected List test(List inList) throws IOException,
                                    InvalidRuleSessionException
    {
        return statelessRuleSession.executeRules( inList );
    }
}